-- RaidAuras based on Bigwigs common auras
--------------------------------------------

local addon = DXE
local L = addon.L

local class_color = {}
for class,color in pairs(RAID_CLASS_COLORS) do
	class_color[class] = ("|cff%02x%02x%02x"):format(color.r * 255, color.g * 255, color.b * 255)
end
-- If needed to go Global class color
local function getclasscolor(unit)
	if unit == nil then return "nil" end
	local classc = select(2,UnitClass(unit))
	if classc == nil and unit ~= nil then return unit end
	return class_color[classc]..unit.."|r" or unit
end

local defaults = {
	profile = {
		Enabled = true,
		Messages = true,
		sound = "ALERT13",
		color1 = "GOLD",
		BarColor = "MIDBLUE",
		--UnendingBreath = false,
		Bloodlust = true,
		Feasts = true,
		Repair = true,
		Soulwell = true,
		SummoningStone = true,
		RallyingCry = true,
		StampedingRoar = true,
		SmokeBomb = true,
		PainSuppression = true,		
		GuardianSpirit = true,
		Barrier = true,
		DivineHymn = true,
		Tranquility = true,		
		Ironbark = true,	
		Sacrifice = true,	
		DevotionAura = true,	
		SpiritLink = true,	
		LifeCocoon = true,	
		Revival = true,	
		AntiMagicZone = true,		
		AmplifyMagic = true,		
		Rebirth = true,					
	},
}
local combatLogMap = {}

combatLogMap.SPELL_CAST_SUCCESS = {
		--OOC
		[22700] = "Repair", -- Field Repair Bot 74A
		[44389] = "Repair", -- Field Repair Bot 110G
		[54711] = "Repair", -- Scrapbot
		[67826] = "Repair", -- Jeeves
		[157066] = "Repair", -- Walter
		[698] = "SummoningStone", -- Ritual of Summoning
		[29893] = "Soulwell", -- Create Soulwell
		--[43987] = "Refreshment", -- Conjure Refreshment Table
		-- Group
		[97462] = "RallyingCry",
		[106898] = "StampedingRoar",
		--[172106] = "AspectOfTheFox",
		-- DPS
		[2825] = "Bloodlust", -- Bloodlust
		[32182] = "Bloodlust", -- Heroism
		[80353] = "Bloodlust", -- Time Warp
		[90355] = "Bloodlust", -- Ancient Hysteria
		[160452] = "Bloodlust", -- Netherwinds
		-- Healing
		[33206] = "PainSuppression",
		[62618] = "Barrier",
		[47788] = "GuardianSpirit",
		[64843] = "DivineHymn",
		[102342] = "Ironbark",
		[740] = "Tranquility",
		[6940] = "Sacrifice",
		[31821] = "DevotionAura",
		[98008] = "SpiritLink",
		[116849] = "LifeCocoon",
		[115310] = "Revival",
		[51052] = "AntiMagicZone",
		[76577] = "SmokeBomb",
		[159916] = "AmplifyMagic",
		-- Test
--[5697] = "UnendingBreath",		
--[21562] = "UnendingBreath",	
}
combatLogMap.SPELL_CAST_START = {
	[160740] = "Feasts", -- Feast of Blood (+75)
	[160914] = "Feasts", -- Feast of the Waters (+75)
	[175215] = "Feasts", -- Savage Feast (+100)
}
combatLogMap.SPELL_RESURRECT = {
	[20484] = "Rebirth", -- Rebirth
	[95750] = "Rebirth", -- Soulstone Resurrection
	[61999] = "Rebirth", -- Raise Ally
	[126393] = "Rebirth", -- Eternal Guardian (Hunter pet)
	[159931] = "Rebirth", -- Gift of Chi-Ji (Hunter pet)
	[159956] = "Rebirth", -- Dust of Life (Hunter pet)
}
--SPELL_CAST_SUCCESS
----------------------------------
-- INITIALIZATION
----------------------------------

local module = addon:NewModule("RaidAuras","AceEvent-3.0", "AceTimer-3.0")
addon.RaidAuras = module

local db,pfl

function module:RefreshProfile() 
	pfl = db.profile

	if not pfl.Enabled then
		module:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		addon.Alerts:QuashByPattern("^raidauras")
	elseif pfl.Enabled then
		module:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	end
end

function module:OnInitialize()
	self.db = addon.db:RegisterNamespace("RaidAuras", defaults)
	db = self.db
	pfl = db.profile

	db.RegisterCallback(self, "OnProfileChanged", "RefreshProfile")
	db.RegisterCallback(self, "OnProfileCopied", "RefreshProfile")
	db.RegisterCallback(self, "OnProfileReset", "RefreshProfile")
	
	if pfl.Enabled then
		self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	end
end
--[[
function module:UnendingBreath(_, spellId, nick, spellName)
print("module",spellId, nick, spellName)
	if pfl.UnendingBreath then
		nick = getclasscolor(nick)
		addon.Alerts:Auras("raidauras"..spellName, nick..": "..spellName, 600, 15, "DXE ALERT2", "MIDBLUE", "GREEN", nil, addon.ST[5697])
		addon.Alerts:NewTMessage(nick.." used "..spellName,pfl.color1,addon.ST[5697],"ALERT13")
	end
end--]]
local prev = 0
function module:Bloodlust(_, spellId, nick, spellName)
	if pfl.Bloodlust then
		local t = GetTime()
		if t-prev > 40 then
			--print("module",spellId, nick, spellName)
			nick = getclasscolor(nick)
			nick = nick:gsub("%-.+", "*")			
			addon.Alerts:Auras("raidauras"..spellName, nick..": "..spellName, 40, 15, "DXE ALERT2", "MIDBLUE", "ORANGE", nil, addon.ST[2825])
			addon.Alerts:NewTMessage(nick..": "..spellName,"GREEN",addon.ST[2825],"ALERT13")
			prev = t
		end
	end
end
function module:Repair(_, spellId, nick, spellName)
	if pfl.Repair then

		SetMapToCurrentZone()
		local x,y = GetPlayerMapPosition(nick)

		--print("Repair",nick,x,y)
		addon.Arrows:AddTarget(nick,6,"TOWARD",spellName,spellName,nil,true,x,y,1,5,20,nil)
		nick = getclasscolor(nick)
		nick = nick:gsub("%-.+", "*")
		if spellId == 54711 then
			addon.Alerts:Auras(spellName.."cd", spellName, 300, 15, "DXE ALERT2", "MIDBLUE", "GREEN", nil, addon.ST[54711])
		elseif spellId == 157066 then
			addon.Alerts:Auras(spellName.."cd", spellName, 360, 15, "DXE ALERT2", "MIDBLUE", "GREEN", nil, addon.ST[157066])
		elseif spellId == 22700 then
			addon.Alerts:Auras(spellName.."cd", spellName, 600, 15, "DXE ALERT2", "MIDBLUE", "GREEN", nil, addon.ST[22700])
		elseif spellId == 44389 then
			addon.Alerts:Auras(spellName.."cd", spellName, 600, 15, "DXE ALERT2", "MIDBLUE", "GREEN", nil, addon.ST[44389])
		elseif spellId == 67826 then
			addon.Alerts:Auras(spellName.."cd", spellName, 600, 15, "DXE ALERT2", "MIDBLUE", "GREEN", nil, addon.ST[67826])
		end	
		addon.Alerts:NewTMessage(nick..": "..spellName,pfl.color1,addon.ST[29893],"ALERT13")
	end
--nick,defn.persist,defn.action,defn.msg,defn.spell,stgs.sound,defn.fixed,
--									 defn.xpos,defn.ypos,defn.range1,defn.range2,defn.range3,defn.lockon)
end
function module:SummoningStone(_, spellId, nick, spellName)
	if pfl.SummoningStone then

		SetMapToCurrentZone()
		local x,y = GetPlayerMapPosition(nick)

		--local x,y = addon:GetDistanceToUnitSetFixed(nick)
		--print("SummoningStone",nick,x,y)	
		addon.Arrows:AddTarget(nick,6,"TOWARD",spellName,spellName,nil,true,x,y,1,5,20,nil)
		nick = getclasscolor(nick)
		nick = nick:gsub("%-.+", "*")
		addon.Alerts:Auras(spellName.."cd", nick..": "..spellName, 180, 15, "DXE ALERT2", "MIDBLUE", "GREEN", nil, addon.ST[698])
	end
end
function module:Feasts(_, spellId, nick, spellName)
	if pfl.Feasts then

		SetMapToCurrentZone()
		local x,y = GetPlayerMapPosition(nick)

		--print("feast",nick,x,y)
		addon.Arrows:AddTarget("player",20,"TOWARD",spellName,spellName,nil,true,x,y,1,5,20,nil)
		nick = getclasscolor(nick)
		nick = nick:gsub("%-.+", "*")
		addon.Alerts:Auras(spellName.."cd", nick..": "..spellName, 180, 15, "DXE ALERT2", "MIDBLUE", "GREEN", nil, addon.ST[160740])
	end	
end
function module:Soulwell(_, spellId, nick, spellName)
	if pfl.Soulwell then

		SetMapToCurrentZone()
		local x,y = GetPlayerMapPosition(nick)

		--print("Soulwell",nick,x,y)
		addon.Arrows:AddTarget(nick,6,"TOWARD",spellName,spellName,nil,true,x,y,1,5,20,nil)
		nick = getclasscolor(nick)
		nick = nick:gsub("%-.+", "*")
		addon.Alerts:Auras(spellName.."cd", nick..": "..spellName, 120, 15, "DXE ALERT2", "MIDBLUE", "GREEN", nil, addon.ST[29893])
		addon.Alerts:NewTMessage(nick..": "..spellName,pfl.color1,addon.ST[29893],"ALERT13")
	end
end
---------------------------------------------------------------------------------------
function module:RallyingCry(_, spellId, nick, spellName)
	if pfl.RallyingCry then
		nick = getclasscolor(nick)
		nick = nick:gsub("%-.+", "*")
		--addon.Alerts:Auras(spellName, nick..": "..spellName, 10, 15, "DXE ALERT2", "MIDBLUE", "RED", nil, addon.ST[97462])
		addon.Alerts:Auras(spellName.."cd", nick..": "..spellName, 180, 15, "DXE ALERT2", "MIDBLUE", "GREEN", nil, addon.ST[97462])
		addon.Alerts:NewTMessage(nick..": "..spellName,pfl.color1,addon.ST[97462],"ALERT13")
	end
end
function module:StampedingRoar(_, spellId, nick, spellName)
	if pfl.StampedingRoar then
		nick = getclasscolor(nick)
		nick = nick:gsub("%-.+", "*")
		--addon.Alerts:Auras(spellName, nick..": "..spellName, 8, 15, "DXE ALERT2", "MIDBLUE", "RED", nil, addon.ST[106898])
		addon.Alerts:Auras(spellName.."cd", nick..": "..spellName, 120, 15, "DXE ALERT2", "MIDBLUE", "GREEN", nil, addon.ST[106898])
		addon.Alerts:NewTMessage(nick..": "..spellName,pfl.color1,addon.ST[106898],"ALERT13")
	end
end
--[[function module:AspectOfTheFox(_, spellId, nick, spellName)
	addon.Alerts:Auras(spellName, nick..": "..spellName, 6, 15, "DXE ALERT2", "MIDBLUE", "RED", nil, addon.ST[172106])
	addon.Alerts:Auras(spellName.."cd", nick..": "..spellName, 120, 15, "DXE ALERT2", "MIDBLUE", "GREEN", nil, addon.ST[172106])
	addon.Alerts:NewTMessage(nick..": "..spellName,pfl.color1,addon.ST[172106],"ALERT13")
end--]]
function module:SmokeBomb(_, spellId, nick, spellName)
	if pfl.SmokeBomb then
		nick = getclasscolor(nick)
		nick = nick:gsub("%-.+", "*")
		--addon.Alerts:Auras(spellName, nick..": "..spellName, 5, 15, "DXE ALERT2", "MIDBLUE", "RED", nil, addon.ST[76577])
		addon.Alerts:Auras(spellName.."cd", nick..": "..spellName, 180, 15, "DXE ALERT2", "MIDBLUE", "GREEN", nil, addon.ST[76577])
		addon.Alerts:NewTMessage(nick..": "..spellName,pfl.color1,addon.ST[76577],"ALERT13")
	end
end
function module:PainSuppression(target, spellId, nick, spellName)
	if pfl.PainSuppression then
		nick = getclasscolor(nick)
		nick = nick:gsub("%-.+", "*")
		--addon.Alerts:Auras(spellName, nick.." -> "..target..": :"..spellName, 8, 15, "DXE ALERT2", "MIDBLUE", "RED", nil, addon.ST[33206])
		addon.Alerts:Auras("raidauras"..spellName, nick..": "..spellName, 180, 15, "DXE ALERT2", "MIDBLUE", "GREEN", nil, addon.ST[33206])
		addon.Alerts:NewTMessage(nick..": "..spellName.." "..L.alert["on"].." "..target,pfl.color1,addon.ST[33206],"ALERT13")
	end
end
function module:GuardianSpirit(target, spellId, nick, spellName)
	if pfl.GuardianSpirit then
		nick = getclasscolor(nick)
		nick = nick:gsub("%-.+", "*")
		--addon.Alerts:Auras(spellName, nick.." -> "..target..": :"..spellName, 10, 15, "DXE ALERT2", "MIDBLUE", "RED", nil, addon.ST[47788])
		addon.Alerts:Auras("raidauras"..spellName, nick..": "..spellName, 180, 15, "DXE ALERT2", "MIDBLUE", "GREEN", nil, addon.ST[47788])
		addon.Alerts:NewTMessage(nick..": "..spellName.." "..L.alert["on"].." "..target,pfl.color1,addon.ST[47788],"ALERT13")
	end
end
function module:Barrier(_, spellId, nick, spellName)
	if pfl.Barrier then
		nick = getclasscolor(nick)
		nick = nick:gsub("%-.+", "*")
		--addon.Alerts:Auras(spellName, nick..": "..spellName, 10, 15, "DXE ALERT2", "MIDBLUE", "RED", nil, addon.ST[62618])
		addon.Alerts:Auras("raidauras"..spellName, nick..": "..spellName, 180, 15, "DXE ALERT2", "MIDBLUE", "GREEN", nil, addon.ST[62618])
		addon.Alerts:NewTMessage(nick..": "..spellName,pfl.color1,addon.ST[62618],"ALERT13")
	end
end
function module:DivineHymn(_, spellId, nick, spellName)
	if pfl.DivineHymn then
		nick = getclasscolor(nick)
		nick = nick:gsub("%-.+", "*")
		--addon.Alerts:Auras(spellName, nick..": "..spellName, 8, 15, "DXE ALERT2", "MIDBLUE", "RED", nil, addon.ST[64843])
		addon.Alerts:Auras("raidauras"..spellName, nick..": "..spellName, 180, 15, "DXE ALERT2", "MIDBLUE", "GREEN", nil, addon.ST[64843])
		addon.Alerts:NewTMessage(nick..": "..spellName,pfl.color1,addon.ST[64843],"ALERT13")
	end
end
function module:Tranquility(_, spellId, nick, spellName)
	if pfl.Tranquility then
		nick = getclasscolor(nick)
		nick = nick:gsub("%-.+", "*")
		--addon.Alerts:Auras(spellName, nick..": "..spellName, 8, 15, "DXE ALERT2", "MIDBLUE", "RED", nil, addon.ST[740])
		addon.Alerts:Auras("raidauras"..spellName, nick..": "..spellName, 180, 15, "DXE ALERT2", "MIDBLUE", "GREEN", nil, addon.ST[740])
		addon.Alerts:NewTMessage(nick..": "..spellName,pfl.color1,addon.ST[740],"ALERT13")
	end
end
function module:Ironbark(target, spellId, nick, spellName)
	if pfl.Ironbark then
		nick = getclasscolor(nick)
		nick = nick:gsub("%-.+", "*")
		--addon.Alerts:Auras(spellName, nick.." -> "..target..": :"..spellName, 12, 15, "DXE ALERT2", "MIDBLUE", "RED", nil, addon.ST[102342])
		addon.Alerts:Auras("raidauras"..spellName, nick..": "..spellName, 60, 15, "DXE ALERT2", "MIDBLUE", "GREEN", nil, addon.ST[102342])
		addon.Alerts:NewTMessage(nick..": "..spellName.." "..L.alert["on"].." "..target,pfl.color1,addon.ST[102342],"ALERT13")
	end
end
function module:Sacrifice(target, spellId, nick, spellName)
	if pfl.Sacrifice then
		nick = getclasscolor(nick)
		nick = nick:gsub("%-.+", "*")
		--addon.Alerts:Auras(spellName, nick.." -> "..target..": :"..spellName, 12, 15, "DXE ALERT2", "MIDBLUE", "RED", nil, addon.ST[6940])
		addon.Alerts:Auras("raidauras"..spellName, nick..": "..spellName, 120, 15, "DXE ALERT2", "MIDBLUE", "GREEN", nil, addon.ST[6940])
		addon.Alerts:NewTMessage(nick..": "..spellName.." "..L.alert["on"].." "..target,pfl.color1,addon.ST[6940],"ALERT13")
	end
end
function module:DevotionAura(_, spellId, nick, spellName)
	if pfl.DevotionAura then
		nick = getclasscolor(nick)
		nick = nick:gsub("%-.+", "*")
		--addon.Alerts:Auras(spellName, nick..": "..spellName, 6, 15, "DXE ALERT2", "MIDBLUE", "RED", nil, addon.ST[31821])
		addon.Alerts:Auras("raidauras"..spellName, nick..": "..spellName, 180, 15, "DXE ALERT2", "MIDBLUE", "GREEN", nil, addon.ST[31821])
		addon.Alerts:NewTMessage(nick..": "..spellName,pfl.color1,addon.ST[31821],"ALERT13")
	end
end
function module:SpiritLink(_, spellId, nick, spellName)
	if pfl.SpiritLink then
		nick = getclasscolor(nick)
		nick = nick:gsub("%-.+", "*")
		--addon.Alerts:Auras(spellName, nick..": "..spellName, 6, 15, "DXE ALERT2", "MIDBLUE", "RED", nil, addon.ST[98008])
		addon.Alerts:Auras("raidauras"..spellName, nick..": "..spellName, 180, 15, "DXE ALERT2", "MIDBLUE", "GREEN", nil, addon.ST[98008])
		addon.Alerts:NewTMessage(nick..": "..spellName,pfl.color1,addon.ST[98008],"ALERT13")
	end
end
function module:LifeCocoon(target, spellId, nick, spellName)
	if pfl.LifeCocoon then
		nick = getclasscolor(nick)
		nick = nick:gsub("%-.+", "*")
		--addon.Alerts:Auras(spellName, nick.." -> "..target..": :"..spellName, 12, 15, "DXE ALERT2", "MIDBLUE", "RED", nil, addon.ST[116849])
		addon.Alerts:Auras("raidauras"..spellName, nick..": "..spellName, 120, 15, "DXE ALERT2", "MIDBLUE", "GREEN", nil, addon.ST[116849])
		addon.Alerts:NewTMessage(nick..": "..spellName.." "..L.alert["on"].." "..target,pfl.color1,addon.ST[116849],"ALERT13")
	end
end
function module:Revival(_, spellId, nick, spellName)
	if pfl.Revival then
		nick = getclasscolor(nick)
		nick = nick:gsub("%-.+", "*")
		addon.Alerts:Auras("raidauras"..spellName, nick..": "..spellName, 180, 15, "DXE ALERT2", "MIDBLUE", "ORANGE", nil, addon.ST[115310])
		addon.Alerts:NewTMessage(nick..": "..spellName,pfl.color1,addon.ST[115310],"ALERT13")
	end
end
function module:AntiMagicZone(_, spellId, nick, spellName)
	if pfl.AntiMagicZone then
		nick = getclasscolor(nick)
		nick = nick:gsub("%-.+", "*")
		--addon.Alerts:Auras(spellName, nick..": "..spellName, 3, 15, "DXE ALERT2", "MIDBLUE", "RED", nil, addon.ST[51052])
		addon.Alerts:Auras("raidauras"..spellName, nick..": "..spellName, 120, 15, "DXE ALERT2", "MIDBLUE", "GREEN", nil, addon.ST[51052])
		addon.Alerts:NewTMessage(nick..": "..spellName,pfl.color1,addon.ST[51052],"ALERT13")
	end
end

function module:AmplifyMagic(_, spellId, nick, spellName)
	if pfl.AmplifyMagic then
		nick = getclasscolor(nick)
		nick = nick:gsub("%-.+", "*")
		--addon.Alerts:Auras(spellName, nick..": "..spellName, 6, 15, "DXE ALERT2", "MIDBLUE", "ORANGE", nil, addon.ST[159916])
		addon.Alerts:Auras("raidauras"..spellName, nick..": "..spellName, 120, 15, "DXE ALERT2", "MIDBLUE", "GREEN", nil, addon.ST[159916])
		addon.Alerts:NewTMessage(nick..": "..spellName,pfl.color1,addon.ST[159916],"ALERT13")
	end
end
function module:Rebirth(target, spellId, nick, spellName)
	if pfl.Rebirth then
		nick = getclasscolor(nick)
		nick = nick:gsub("%-.+", "*")
		addon.Alerts:Auras("raidauras"..spellName, nick..": "..spellName, 600, 15, "DXE ALERT2", "MIDBLUE", "GREEN", nil, addon.ST[20484])
		addon.Alerts:NewTMessage(nick..": "..spellName.." "..L.alert["on"].." "..target,pfl.color1,addon.ST[20484],"ALERT13")
	end
end

function module:COMBAT_LOG_EVENT_UNFILTERED(a, b, event, c, d, source, e, f, g, player, h, i, spellId, spellName)
	local f = combatLogMap[event] and combatLogMap[event][spellId] or nil
	
	--if player ~= UnitName("player") then return end
	--print("ad",spellId, spellName)
	if f and player then
		self[f](self, player, spellId, source, spellName)
	elseif f then
		self[f](self, player, spellId, source, spellName)
	end
	--print("sss",a, b, event, c, d, source, e, f, g, player, h, i, spellId, spellName)

end

