--------------------------------------------
--[[
DXE Radar - a graphical proximity check.
Some of the code is straight from BigWigs. Many thanks and credits to them.
This textures used are copyright BigWigs: alert_circle.blp and dot.tga
]]
--------------------------------------------
local addon = DXE
local L = addon.L
local name_to_unit = addon.Roster.name_to_unit
local name_to_class = addon.Roster.name_to_class

local window
local pfl

--Radial upvalues
local GetPlayerMapPosition = GetPlayerMapPosition
local UnitPosition = UnitPosition
local GetPlayerFacing = GetPlayerFacing
local format = string.format
local UnitInRange = UnitInRange
local UnitIsDead = UnitIsDead
local UnitIsUnit = UnitIsUnit
local UnitClass = UnitClass
local GetTime = GetTime
local min = math.min
local pi = math.pi
local cos = math.cos
local sin = math.sin
local tremove = table.remove
local unpack = unpack
local activeMap
local proximityPlayer = nil
local proximityPlayerTable = {}
local normalproximity = true
local multiproximity = false

local configmode = false

local titleHeight = 12
local titleBarInset = 2

local DOTSIZE
local ICONBEHAVE

local unitList = nil
local raidList = {}
local partyList = {}
local maxPlayers = 0
local myGUID = nil

do
	local raidList = {
		"raid1", "raid2", "raid3", "raid4", "raid5", "raid6", "raid7", "raid8", "raid9", "raid10",
		"raid11", "raid12", "raid13", "raid14", "raid15", "raid16", "raid17", "raid18", "raid19", "raid20",
		"raid21", "raid22", "raid23", "raid24", "raid25", "raid26", "raid27", "raid28", "raid29", "raid30",
		"raid31", "raid32", "raid33", "raid34", "raid35", "raid36", "raid37", "raid38", "raid39", "raid40"
	}
	function addon:GetRaidList()
		return raidList
	end
end

do
	local partyList = {"player", "party1", "party2", "party3", "party4"}
	function addon:GetPartyList()
		return partyList
	end
end
---------------------------------------
-- UTILITY
---------------------------------------
local function RadarSize()

	if window then
		local content = window.content
		local container = window.container
		local faux_window = window.faux_window
		container:ClearAllPoints()
		container:SetPoint("TOPLEFT",faux_window,"TOPLEFT",1,-titleHeight-titleBarInset)
		container:SetPoint("BOTTOMRIGHT",faux_window,"BOTTOMRIGHT",-1,1)
		container:SetSize(faux_window:GetWidth()-1,faux_window:GetHeight()-titleHeight-titleBarInset)
		window:SetContentInset(1)
		content:SetSize(faux_window:GetWidth()-1,faux_window:GetHeight()-titleHeight-titleBarInset)

		local scale = faux_window:GetScale()
		local range = pfl.Proximity.Range
		local w,h = content:GetSize()
		local ppy = min(w, h) / (range * 3)
		content.rangeCircle:SetSize(ppy * range * 2 * scale, ppy * range * 2 * scale)
	end
end
local function OnSize(...)
	if window then
		RadarSize()
		if configmode then
			testDots()
		end
	end
	return true
end

local hexColors = {}
local vertexColors = {}
for k, v in pairs(RAID_CLASS_COLORS) do
	hexColors[k] = ("|cff%02x%02x%02x"):format(v.r * 255, v.g * 255, v.b * 255)
	vertexColors[k] = { v.r, v.g, v.b }
end

---------------------------------------
-- BARS
---------------------------------------
local bars = {}
local bar_pool = {}

---------------------------------------
-- WINDOW CREATION
--------------------------------------
local ProximityFuncs = addon:GetProximityFuncs()
local range -- yds
local activeRangeSquared
local invert
local delay
local rangefunc -- proximity function

local OnUpdate


local function OnShow(self)
	counter = 0
end

local function OnHide(self)
	for i,bar in ipairs(bars) do return true end
end

local function UpdateTitle()
	window:SetTitle(format("%s - %d",L["Radar"],range))
end

local function OpenOptions()
	addon:ToggleConfig()
	if not addon.Options then return end
	if LibStub("AceConfigDialog-3.0").OpenFrames.DXE then LibStub("AceConfigDialog-3.0"):SelectGroup("DXE","windows_group","proximity_group") end
end

local RadarDropDownMenu = CreateFrame("Frame", "RadarDropDownMenu")
RadarDropDownMenu.displayMode = "MENU"
RadarDropDownMenu.info = {}
RadarDropDownMenu.UncheckHack = function(dropdownbutton)
    _G[dropdownbutton:GetName().."Check"]:Hide()
end
RadarDropDownMenu.HideMenu = function()
    if UIDROPDOWNMENU_OPEN_MENU == RadarDropDownMenu then
        CloseDropDownMenus()
    end
end

RadarDropDownMenu.initialize = function(self, level)
    if not level then return end
    local info = self.info
    wipe(info)
    if level == 1 then
        info.isTitle = 1
        info.text = "Radar"
        info.notCheckable = 1
        UIDropDownMenu_AddButton(info, level)

        info.keepShownOnClick = 1
        info.disabled = nil
        info.isTitle = nil
        info.notCheckable = nil

        info.text = "Range"
        info.func = self.UncheckHack
        info.hasArrow = 1
		info.isNotRadio = 1
		info.notCheckable = true
        info.value = "submenu1"
        UIDropDownMenu_AddButton(info, level)

        --info.text = "Wxyz" -- Note .hasArrow and .func fallthrough from prev item.
        --info.value = "submenu2"
        --UIDropDownMenu_AddButton(info, level)

        -- Close menu item
        info.hasArrow     = nil
        info.value        = nil
        info.notCheckable = 1
        info.text         = CLOSE
        info.func         = self.HideMenu
        UIDropDownMenu_AddButton(info, level)

    elseif level == 2 then
        if UIDROPDOWNMENU_MENU_VALUE == "submenu1" then
            info.text = "2 yards"
			info.func = function()
				pfl.Proximity.Range = 2
				addon:UpdateProximityProfile()
				RadarDropDownMenu.HideMenu()
			end
			info.checked = (pfl.Proximity.Range == 2)

            UIDropDownMenu_AddButton(info, level)

            info.text = "3 yards"
			info.func = function()
				pfl.Proximity.Range = 3
				addon:UpdateProximityProfile()
				RadarDropDownMenu.HideMenu()
			end
			info.checked = (pfl.Proximity.Range == 3)
            UIDropDownMenu_AddButton(info, level)

            info.text = "4 yards"
			info.func = function()
				pfl.Proximity.Range = 4
				addon:UpdateProximityProfile() 
				RadarDropDownMenu.HideMenu()
			end
			info.checked = (pfl.Proximity.Range == 4)
            UIDropDownMenu_AddButton(info, level)
			
            info.text = "5 yards"
			info.func = function()
				pfl.Proximity.Range = 5
				addon:UpdateProximityProfile() 
				RadarDropDownMenu.HideMenu()
			end
			info.checked = (pfl.Proximity.Range == 5)
            UIDropDownMenu_AddButton(info, level)
			
            info.text = "6 yards"
			info.func = function()
				pfl.Proximity.Range = 6
				addon:UpdateProximityProfile() 
				RadarDropDownMenu.HideMenu()
			end
			info.checked = (pfl.Proximity.Range == 6)
            UIDropDownMenu_AddButton(info, level)
			
            info.text = "8 yards"
			info.func = function()
				pfl.Proximity.Range = 8
				addon:UpdateProximityProfile() 
				RadarDropDownMenu.HideMenu()
			end
			info.checked = (pfl.Proximity.Range == 8)
            UIDropDownMenu_AddButton(info, level)
			
            info.text = "10 yards"
			info.func = function()
				pfl.Proximity.Range = 10
				addon:UpdateProximityProfile() 
				RadarDropDownMenu.HideMenu()
			end
			info.checked = (pfl.Proximity.Range == 10)
            UIDropDownMenu_AddButton(info, level)
			
            info.text = "12 yards"
			info.func = function()
				pfl.Proximity.Range = 12
				addon:UpdateProximityProfile() 
				RadarDropDownMenu.HideMenu()
			end
			info.checked = (pfl.Proximity.Range == 12)
            UIDropDownMenu_AddButton(info, level)
			
            info.text = "15 yards"
			info.func = function()
				pfl.Proximity.Range = 15
				addon:UpdateProximityProfile() 
				RadarDropDownMenu.HideMenu()
			end
			info.checked = (pfl.Proximity.Range == 15)
            UIDropDownMenu_AddButton(info, level)
			
            info.text = "18 yards"
			info.func = function()
				pfl.Proximity.Range = 18
				addon:UpdateProximityProfile() 
				RadarDropDownMenu.HideMenu()
			end
			info.checked = (pfl.Proximity.Range == 18)
            UIDropDownMenu_AddButton(info, level)
			
			
			
       --[[ elseif UIDROPDOWNMENU_MENU_VALUE == "submenu2" then
            info.text = "Moo"
            UIDropDownMenu_AddButton(info, level)

            info.text = "Lar"
            UIDropDownMenu_AddButton(info, level)
--]]
        end
    end
end

local function CreateWindow()
	window = addon:CreateWindow(L["Radar"],150,150)
	window:SetMinResize(100, 30)
	window:SetClampedToScreen(true)
	window:Hide()
	window:SetContentInset(1)

	window:RegisterCallback("OnSizeChanged",OnSize)
	window:RegisterCallback("OnScaleChanged",OnSize)
	window:SetScript("OnUpdate",OnUpdate)
	window:SetScript("OnShow",OnShow)
	window:SetScript("OnHide",OnHide)

	window:AddTitleButton("Interface\\AddOns\\DXE\\Textures\\Pane\\Menu.tga",OpenOptions,L["Options"])

	window:SetScript("OnMouseUp", function(self, btn)
        if btn == "RightButton" then
			ToggleDropDownMenu(1, nil, RadarDropDownMenu, 'cursor', 0, 0)
		end
	end)
	local faux_window = window.faux_window
	local container = window.container
	container:ClearAllPoints()
	container:SetPoint("TOPLEFT",faux_window,"TOPLEFT",1,-titleHeight-titleBarInset)
	container:SetPoint("BOTTOMRIGHT",faux_window,"BOTTOMRIGHT",-1,1)
	container:SetSize(faux_window:GetWidth()-1,faux_window:GetHeight()-titleHeight-titleBarInset)

	local content = window.content
	content:SetSize(faux_window:GetWidth()-1,faux_window:GetHeight()-titleHeight-titleBarInset)

	local rangeCircle = window:CreateTexture(nil, "ARTWORK", content)
	rangeCircle:SetTexture([[Interface\AddOns\DXE\Textures\alert_circle]])
	rangeCircle:SetPoint("CENTER",content)
	rangeCircle:SetBlendMode("ADD")
	local scale = faux_window:GetScale()
	--local range = pfl.Proximity.Range
	local range2 = range
	local w,h = content:GetSize()
	local ppy = min(w, h) / (range2 * 3)
	rangeCircle:SetSize(ppy * range2 * 2 * scale, ppy * range2 * 2 * scale)
	content.rangeCircle = rangeCircle

	local playerDot = window:CreateTexture(nil, "OVERLAY", content)
	playerDot:SetTexture([[Interface\Minimap\MinimapArrow]])
	playerDot:SetSize(32, 32)
	playerDot:SetBlendMode("ADD")
	playerDot:SetPoint("CENTER",content)
	content.playerDot = playerDot

	window:Show()
	CreateWindow = nil

	addon:UpdateRadarSettings()
end

---------------------------------------
-- UPDATER
--------------------------------------

local updater = nil
local graphicalUpdater = nil

do
	local proxDots = {}
	local cacheDots = {}
	local cacheIcons = {}

	function addon:DotResize(dotsize)
		DOTSIZE = dotsize
		if cacheDots then
			for i=1,#cacheDots do
				cacheDots[i]:SetSize(DOTSIZE,DOTSIZE)
			end
		end
		if proxDots then
			for i=1,#proxDots do
				proxDots[i]:SetSize(DOTSIZE,DOTSIZE)
			end
		end
		if cacheIcons then
			for i=1,8 do
				if cacheIcons[i] then
					cacheIcons[i]:SetSize(DOTSIZE,DOTSIZE)
				end
			end
		end

	end

	function addon:dotter()
		if window then
			if not configmode then
				window:SetScript("OnUpdate",nil)
				testDots()
				configmode = true
			else
				window:SetScript("OnUpdate",OnUpdate)
				hideDots()
				window.content.rangeCircle:SetVertexColor(0, 1, 0)
				configmode = false
			end
		end
	end

	--local lastplayed = 0 -- When we last played an alarm sound for proximity.

	-- dx and dy are in yards
	-- class is player class
	-- facing is radians with 0 being north, counting up clockwise
	setDot = function(dx, dy, class, icon)
		local content = window.content
		local width, height = content:GetSize()
		local range2 = tonumber(range) > 0 and tonumber(range) or 10
		-- range * 3, so we have 3x radius space
		local pixperyard = min(width, height) / (range2 * 3)

		-- rotate relative to player facing
		--local rotangle = (2 * pi) - GetPlayerFacing()
		--local x = (dx * cos(rotangle)) - (-1 * dy * sin(rotangle))
		--local y = (dx * sin(rotangle)) + (-1 * dy * cos(rotangle))
		
		-- rotate relative to player facing
		local rotangle = (2 * pi) - GetPlayerFacing()
		local x = (-1 * dx * cos(rotangle)) - (dy * sin(rotangle))
		local y = (-1 * dx * sin(rotangle)) + (dy * cos(rotangle))		
		
		x = x * pixperyard
		y = y * pixperyard

		-- Clamp to frame if out-of-bounds
		if x < -(width / 2) then
			x = -(width / 2)
		elseif x > (width / 2) then
			x = (width / 2)
		end
		if y < -(height / 2) then
			y = -(height / 2)
		elseif y > (height / 2) then
			y = (height / 2)
		end	
		
		local dot = nil
		if #cacheDots > 0 then
			dot = tremove(cacheDots)
		else
			dot = content:CreateTexture(nil, "OVERLAY")
			dot:SetSize(DOTSIZE,DOTSIZE)
			dot:SetTexture([[Interface\AddOns\DXE\Textures\dot]])
		end
		proxDots[#proxDots + 1] = dot

		dot:ClearAllPoints()
		dot:SetPoint("CENTER", content, "CENTER", x, y)
		dot:SetVertexColor(unpack(vertexColors[class]))
		dot:Show()

		-- add icon if marked
		if icon and icon > 0 and icon < 9 and ICONBEHAVE ~= "NONE" then
			if not cacheIcons[icon] then
				local iconframe = content:CreateTexture(nil, "OVERLAY")
				iconframe:SetTexture(format([[Interface\TARGETINGFRAME\UI-RaidTargetingIcon_%d.blp]], icon))
				iconframe:SetSize(DOTSIZE,DOTSIZE)
				cacheIcons[icon] = iconframe
			end
			local iconframe = cacheIcons[icon]
			iconframe:ClearAllPoints()
			iconframe:SetPoint("CENTER", content, "CENTER", x, y)
			iconframe:SetDrawLayer("OVERLAY", 1)
			if ICONBEHAVE == "ABOVE" then
				iconframe:Show()
			elseif ICONBEHAVE == "REPLACE" then
				dot:Hide()
				iconframe:Show()
			end
		end
	end

	hideDots = function()
		-- shuffle existing dots into cacheDots
		-- hide those cacheDots
		while #proxDots > 0 do
			proxDots[1]:Hide()
			cacheDots[#cacheDots + 1] = tremove(proxDots, 1)
		end
		-- hide marks
		for i=1,8 do
			if cacheIcons[i] then
				cacheIcons[i]:Hide()
			end
		end
	end

	testDots = function()
		local content = window.content
		hideDots()
		setDot(8, 4, "WARLOCK", 0)
		setDot(2, 10, "HUNTER", 0)
		setDot(-10, -4, "MAGE", 7)
		setDot(0, -8, "PRIEST", 0)
		RadarSize()
		content.rangeCircle:SetVertexColor(1,0,0)
		content.rangeCircle:Show()
		content.playerDot:Show()
	end
	
	local function normalProximityRadar()
		local content = window.content
		local anyoneClose = 0

		local srcY, srcX, _, mapId = UnitPosition("player")
		hideDots()
		for i = 1, maxPlayers do
			local n = unitList[i]
			local unitY, unitX, _, tarMapId = UnitPosition(n)
			local dx = unitX - srcX
			local dy = unitY - srcY
			--local range = dx * dx + dy * dy
			local prange = (dx * dx + dy * dy) ^ 0.5
			if mapId == tarMapId and prange < range*1.1 then
				if myGUID ~= UnitGUID(n) and not UnitIsDead(n) then
					local _,class= UnitClass(n)
					setDot(dx, dy, class, GetRaidTargetIndex(n))
					if tonumber(prange) <= tonumber(range) then
						anyoneClose = anyoneClose + 1
					end
				--elseif blipList[n].isShown then -- A unit may die next to us
				--	blipList[n]:Hide()
				--	blipList[n].isShown = nil
				end
			--elseif blipList[n].isShown then
			--	blipList[n]:Hide()
			--	blipList[n].isShown = nil
			end
		end

		--proxTitle:SetFormattedText(L_proximityTitle, activeRange, anyoneClose)

		if anyoneClose == 0 then
			content.rangeCircle:SetVertexColor(0, 1, 0)
		else
			content.rangeCircle:SetVertexColor(1, 0, 0)
			--if not db.sound then return end
			--local t = GetTime()
			--if t > (lastplayed + db.soundDelay) and not UnitIsDead("player") and InCombatLockdown() then
		--		lastplayed = t
		--		plugin:SendMessage("BigWigs_Sound", db.soundName, true)
		--	end
		end
	end

	local function multiProximityRadar()
		local content = window.content
		--[[local srcX, srcY = GetPlayerMapPosition("player")
		if srcX == 0 and srcY == 0 then
			SetMapToCurrentZone()
			srcX, srcY = GetPlayerMapPosition("player")
		end

		local id = nil
		if activeMap then
			local currentFloor = GetCurrentMapDungeonLevel()
			if currentFloor == 0 then currentFloor = 1 end
			id = activeMap[currentFloor]
		end--]]

		local anyoneClose = nil
			
		-- XXX We can't show/hide dots every update, that seems excessive.
		hideDots()
		local srcY, srcX = UnitPosition("player")
		for i = 1, #proximityPlayerTable do
			local player = proximityPlayerTable[i]
			local unitY, unitX = UnitPosition(player)
			local dx = unitX - srcX
			local dy = unitY - srcY
			local prange = (dx * dx + dy * dy) ^ 0.5
			local _,class= UnitClass(player)
			setDot(dx, dy, class, GetRaidTargetIndex(player))
			if tonumber(prange) <= tonumber(range) then
				anyoneClose = true
			end
		end
--[[
		local srcY, srcX = UnitPosition("player")
		for i = 1, #proximityPlayerTable do
			local player = proximityPlayerTable[i]
			if UnitInRange(player) and not UnitIsDead(player) and not UnitIsUnit(player, "player") then
				local class = select(2,UnitClass(player))
				if pfl.Proximity.ClassFilter[class] then
					local unitY, unitX = UnitPosition(player)
					local dx = unitX - srcX
					local dy = unitY - srcY
					local prange = (dx * dx + dy * dy) ^ 0.5
					if tonumber(prange) < tonumber(range) then
						local _,class= UnitClass(player)
						setDot(dx, dy, class, GetRaidTargetIndex(player))
						--if prange <= range then
							anyoneClose = true
						--end
					end
				end
			end
		end--]]
		if not anyoneClose then
			--lastplayed = 0
			content.rangeCircle:SetVertexColor(0, 1, 0)
		else
			content.rangeCircle:SetVertexColor(1, 0, 0)
		end
	end
	
	local function targetProximityRadar()
		local content = window.content
		--[[local srcX, srcY = GetPlayerMapPosition("player")
		if srcX == 0 and srcY == 0 then
			SetMapToCurrentZone()
			srcX, srcY = GetPlayerMapPosition("player")
		end

		local id = nil
		if activeMap then
			local currentFloor = GetCurrentMapDungeonLevel()
			if currentFloor == 0 then currentFloor = 1 end
			id = activeMap[currentFloor]
		end--]]
		local srcY, srcX = UnitPosition("player")
		local unitY, unitX = UnitPosition(proximityPlayer)
		
		local anyoneClose = nil

		-- XXX We can't show/hide dots every update, that seems excessive.
		hideDots()
		--local unitX, unitY = GetPlayerMapPosition(proximityPlayer)
		local dx = unitX - srcX
		local dy = unitY - srcY
		local prange = (dx * dx + dy * dy) ^ 0.5
		local _,class= UnitClass(proximityPlayer)
		setDot(dx, dy, class, GetRaidTargetIndex(proximityPlayer))
		if tonumber(prange) <= tonumber(range) then
			anyoneClose = true
		end
		--[[if prange <= range and not UnitIsDeadOrGhost(proximityPlayer) and not UnitIsUnit(proximityPlayer, "player") then
			local _,class= UnitClass(proximityPlayer)
			if pfl.Proximity.ClassFilter[class] then
				setDot(dx, dy, class, GetRaidTargetIndex(proximityPlayer))
				if tonumber(prange) <= tonumber(range) then
					anyoneClose = true
				end
			end
		end--]]

		if not anyoneClose then
			--lastplayed = 0
			content.rangeCircle:SetVertexColor(0, 1, 0)
		else
			content.rangeCircle:SetVertexColor(1, 0, 0)
		end
	end
	
	local counter = 0
	local delay = 0.05

	-- 20x per second for radar mode
	function OnUpdate(self, elapsed)
		if delay > 0 then
			counter = counter + elapsed
			if counter < delay then return end
		end
		counter = 0
		if normalproximity == true then
			normalProximityRadar()
		elseif multiproximity == true then
			multiProximityRadar()
		elseif proximityPlayer then
			targetProximityRadar()
		end
	end
end


---------------------------------------
-- API
---------------------------------------


function addon:Radar(popup,enc_range,players,multi)
	if popup and not pfl.Proximity.AutoPopup then return end
	proximityPlayerTable = {}
	proximityPlayer = nil
	multiproximity = false
	
	myGUID = UnitGUID("player")
	
	if players then
		--for i = 1, #players do
		--	print("radar2",players[i],i)
		--end
		--print("radar1111",type(players),players,multi)
	--[[	addon:GetRaidList()
				local idTable = {strsplit(" ", players)}
				for i = 1, #idTable do
					local unitId = (IsInRaid() and "raid") or "party"
					for j = 1, GetNumGroupMembers() do
					print("radar111111",idTable[i],raidList[j],unitId..j)
						--if UnitIsUnit(idTable[i], raidList[j]) then
							--print("radar3",idTable[i])
						--	proximityPlayerTable[#proximityPlayerTable+1] = raidList[j]
						--	break
					--	end						
					end
				end--]]
		normalproximity = false
		--if type(players) == "table" then
		if multi then
			--print("radar",type(players),players,multi)
			multiproximity = true
			local unitId = (IsInRaid() and "raid") or "party"
			if IsInRaid() then
				--addon:GetRaidList()
				--players = "teste1 teste2 teste3 teste4 teste5"
				local idTable = {strsplit(" ", players)}
				local unitId = (IsInRaid() and "raid") or "party"
				for i = 1, #idTable do
					--print("radar2",idTable[i])
					for j = 1, GetNumGroupMembers() do
						if UnitIsUnit(idTable[i], unitId..j) then
							--print("radar3",idTable[i])
							proximityPlayerTable[#proximityPlayerTable+1] = unitId..j
							break
						end						
					end
				end
				--[[for j = 1, GetNumGroupMembers() do
					local pl = {strsplit(" ", players or "")}
					for i, pl in pairs(pl) do				
						if UnitIsUnit(pl[i], raidList[j]) then
							print("radar2",pl)
							proximityPlayerTable[#proximityPlayerTable+1] = raidList[j]
							break
						end
					end					
				end--]]
				--[[for i = 1, #players do
					for j = 1, GetNumGroupMembers() do
						print("sdfsdfsdf",players[i], raidList[j])
						if UnitIsUnit(players[i], raidList[j]) then
							proximityPlayerTable[#proximityPlayerTable+1] = raidList[j]
							break
						end
					end
				end	
				for i,v in ipairs(players) do
					print("asdasdads1",v)
					--s = s..v.." " 
					--sa = sa + 1 
				end --]]
				--[[local pl = {strsplit(" ", players or "")}
				for i, pl in pairs(pl) do
					for j = 1, GetNumGroupMembers() do
						--if UnitIsUnit(pl[i], raidList[j]) then
							print("radar2",pl[i],raidList[j])
							--proximityPlayerTable[#proximityPlayerTable+1] = raidList[j]
							break
						--end
					end
				end		--]]			
			else
			--	addon:GetPartyList()
				for j = 1, GetNumGroupMembers() do
					local pl = {strsplit(" ", players or "")}
					for i, pl in pairs(pl) do				
						if UnitIsUnit(pl[i], unitId..j) then
							proximityPlayerTable[#proximityPlayerTable+1] = unitId..j
							break
						end
					end					
				end					
				--[[for i = 1, #players do
					for j = 1, GetNumGroupMembers() do
						if UnitIsUnit(players[i], partyList[j]) then
							proximityPlayerTable[#proximityPlayerTable+1] = partyList[j]
							break
						end
					end
				end	--]]			
			end
		else
			proximityPlayer = players 
		end
	else
		normalproximity = true
	end
	--print("radar",players)
	if window then window:Show()
	else CreateWindow() end
	unitList = IsInRaid() and addon:GetRaidList() or addon:GetPartyList()
	range = enc_range or pfl.Proximity.Range
	activeRangeSquared = range*range
	UpdateTitle()
	SetMapToCurrentZone()
	--[[activeMap = mapData[(GetMapInfo())]

	if not activeMap then
		SetMapToCurrentZone()
		local floor, a1, b1, c1, d1 = GetCurrentMapDungeonLevel()
		if not (a1 and b1 and c1 and d1) then
			local zoneIndex, a2, b2, c2, d2 = GetCurrentMapZone()
			a1, b1, c1, d1 = a2, b2, c2, d2
		end	
		if not (a1 and b1 and c1 and d1) then return end
		activeMap = {abs(c1-a1), abs(d1-b1)}
	end--]]

	--if activeMap then
		RadarSize()
		local content = window.content
		content.playerDot:Show()
		content.rangeCircle:Show()
	--end
end

function addon:HideRadar()
	if window then window:Hide() end
end

addon:RegisterWindow(L["Radar"],function() addon:Radar() end)

local module = CreateFrame("Frame")

function module:GROUP_ROSTER_UPDATE()
	maxPlayers = GetNumGroupMembers()
	unitList = IsInRaid() and addon:GetRaidList() or addon:GetPartyList()
end


module:SetScript("OnEvent",function(self,event,...) self[event](self,...) end)
module:RegisterEvent("GROUP_ROSTER_UPDATE")
---------------------------------------
-- SETTINGS
---------------------------------------

function addon:UpdateRadarSettings()
	addon:HideProximity()
	rows = pfl.Proximity.Rows
	unitList = IsInRaid() and addon:GetRaidList() or addon:GetPartyList()
	range = pfl.Proximity.Range
	activeRangeSquared = range*range
	delay = pfl.Proximity.Delay
	invert = pfl.Proximity.Invert
	DOTSIZE = pfl.Proximity.DotSize
	ICONBEHAVE = pfl.Proximity.RaidIcons
	rangefunc = range <= 10 and ProximityFuncs[10] or (range <= 11 and ProximityFuncs[11] or ProximityFuncs[18])

	if window then
		UpdateTitle()
		RadarSize()
		addon:DotResize(DOTSIZE)
		if (pfl.Proximity.Dummy and not configmode) or (not pfl.Proximity.Dummy and configmode) then
			addon:dotter()
		elseif pfl.Proximity.Dummy and configmode then --bad hack
			addon:dotter()
			--addon:dotter()
		end
	end
end

local function RefreshProfile(db)
	pfl = db.profile
	addon:UpdateRadarSettings()
end
addon:AddToRefreshProfile(RefreshProfile)
